import json
import unittest
from unittest.mock import patch

from src.dhuolib import Dhuolib


class TestDhuolibUtils(unittest.TestCase):
    def setUp(self):
        self.dhuolib = Dhuolib(service_endpoint="http://localhost:8000")
        self.namespace = "engdb"
        self.bucket_name = "dhuodata-mlops"
        self.file_path = "tests/files/LogisticRegression_best.pickle"
        self.experiment_id = None

    def tearDown(self):
        pass

    @patch("requests.post")
    def test_create_experiment(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 200
        mock_response.json.return_value = {"experiment_id": "1"}

        experiments_params = {
            "experiment_name": "sentiment_analysis_model_1",
            "tags": {"version": "v1", "priority": "P1"},
            "file": {
                "bucket": self.bucket_name,
                "namespace": self.namespace,
                "filepath": self.file_path,
            },
        }
        response = self.dhuolib.create_experiment(experiments_params)

        self.assertEqual(response, mock_response.json.return_value)

        mock_post.assert_called_once_with(
            "http://localhost:8000/api/experiment/save",
            data=json.dumps(experiments_params),
            headers={"Content-Type": "application/json"},
        )

    @patch("requests.post")
    def test_run_experiment(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "current_stage": "Production",
            "last_updated_timestamp": 1713582060414,
            "model_version": "1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "status": "READY",
        }

        run_params = {
            "experiment_id": "2",
            "stage": "Production",
            "file": {
                "bucket": self.bucket_name,
                "namespace": self.namespace,
                "filepath": "LogisticRegression_best.pickle",
            },
        }
        response = self.dhuolib.run_experiment(run_params)

        self.assertEqual(response, mock_response.json.return_value)

        mock_post.assert_called_once_with(
            "http://localhost:8000/api/experiment/run",
            data=json.dumps(run_params),
            headers={"Content-Type": "application/json"},
        )



import oci


class OCIUtils:
    def __init__(self, config_file_path="~/.oci/config", passphrase=None):
        if not config_file_path:
            raise ValueError("config_file_path is required")

        config = oci.config.from_file(config_file_path)
        if passphrase:
            config["pass_phrase"] = passphrase

        self.client = oci.object_storage.ObjectStorageClient(config)

    def upload_file(self, namespace: str, bucket_name: str, file_path: str):
        with open(file_path, "rb") as file:
            object_name = file_path.split("/")[-1]
            self.client.put_object(namespace, bucket_name, object_name, file)
        return f"File {object_name} uploaded to bucket {bucket_name}"



# import unittest
# from unittest import mock
# import pickle

# from src.services import ServiceAPIML


# class TestServiceAPIML(unittest.TestCase):
#     def setUp(self):
#         self.service_endpoint = "http://localhost:5000"
#         self.service = ServiceAPIML(self.service_endpoint)
#         data = {'key': 'value', 'number': 42}

#         with open('tests/files/data.pickle', 'wb') as file:
#             pickle.dump(data, file)

#     @mock.patch('src.services.requests.post')
#     def test_send_post_with_pickle_and_json(self, mock_post):
#         json_data = {
#             "experiment_name": "test_experiment",
#             "model_name": "test_model",
#             "tag": "1.0.0",
#             "file_path": "data.pickle.pkl"
#         }
#         pickle_path = "tests/files/data.pickle"

#         expected_response = mock.Mock()
#         expected_response.status_code = 201

#         mock_post.return_value = expected_response

#         response = self.service.send_post_with_pickle_and_json(json_data, pickle_path)

#         self.assertEqual(response.status_code, 201)



# https://setuptools.pypa.io/en/latest/userguide/declarative_config.html
import os

from setuptools import find_packages, setup

REQUIRED_PACKAGES = open("requirements.txt").readlines()

DEV_PACKAGES = []
if os.path.exists("requirements.dev.txt"):
    DEV_PACKAGES = open("requirements.dev.txt").readlines()

README = ""
if os.path.exists("README.md"):
    README = open("README.md").read()

setup(
    name="dhuolib-tools",
    version="0.0.7",
    long_description=README,
    long_description_content_type="text/markdown",
    author="DHuO Data Team",
    author_email="diego.salles@engdb.com.br",
    url="https://gitlab.engdb.com.br/dhuo-plat/dhuo-data/data-science/dhuolib",
    install_requires=REQUIRED_PACKAGES,
    extras_require={"interactive": DEV_PACKAGES},
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    platforms="any",
)
